<?php
/**********************************************************************************************
*                            CMS Open Business Card
*                              -----------------
*	version				:	%TAG%
*	copyright			:	(c) 2012 Monoray
*	website				:	http://www.monoray.ru/
*	contact us			:	http://www.monoray.ru/contact
*
* This file is part of CMS Open Business Card
*
* Open Business Card is free software. This work is licensed under a GNU GPL.
* http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
*
* Open Business Card is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* Without even the implied warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
***********************************************************************************************/



class Menu extends ParentModel {	
	const LINK_NEW_MANUAL = 1; 
	const LINK_NEW_AUTO = 2;
	const LINK_DROPDOWN = 3;
	const LINK_DROPDOWN_MANUAL = 4;
	const LINK_DROPDOWN_AUTO = 5;

	public static function model($className=__CLASS__){
		return parent::model($className);
	}

	public function tableName(){
		return '{{menu}}';
	}

	public function rules(){
		return array(
			array('type', 'required'),
			
			array('menu_title, href, subitems', 'required', 'on' => 'insert, update'),

			array('menu_title, href', 'required', 'on' => 'link_'.self::LINK_NEW_MANUAL),
			array('menu_title', 'required', 'on' => 'link_'.self::LINK_DROPDOWN),
			array('menu_title, href, subitems', 'required', 'on' => 'link_'.self::LINK_DROPDOWN_MANUAL),

			array('menu_title', 'required', 'on' => 'special'),

			array('page_title, page_body, widget, seo_title, seo_keywords, seo_description', 'safe'),
		);
	}

	public function attributeLabels(){
		return array(
			'id' => 'ID',
			'active' => 'Статус',
			'menu_title' => 'Заголовок меню',
			'page_title' => 'Заголовок страницы',
			'page_body' => 'Текст страницы',

			'seo_title' => 'Title страницы (для SEO)',
			'seo_keywords' => 'Keywords страницы (для SEO)',
			'seo_description' => 'Description страницы (для SEO)',
			'type' => 'Тип ссылки',
			'href' => 'Ссылка',
			'subitems' => 'Выпадающий список для размещения в нем',
			'widget' => 'Отобразить снизу страницы',
		);
	}

	public function search(){
		$criteria=new CDbCriteria;
		$criteria->compare('active', $this->active,true);
		$criteria->compare('menu_title', $this->menu_title,true);
		$criteria->compare('page_title', $this->page_title,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
			'sort' => array(
				'defaultOrder' => 'sorter ASC',
			),
			'pagination'=>array(
				'pageSize'=>param('adminPaginationPageSize', 20),
			),
		));
	}
	
	public function behaviors(){
		return array(
			'AutoTimestampBehavior' => array(
				'class' => 'zii.behaviors.CTimestampBehavior',
				'createAttribute' => 'date_updated',
				'updateAttribute' => 'date_updated',
			),
		);
	}

	public function beforeDelete(){
		$sql = 'UPDATE {{menu}} SET subitems=0 WHERE subitems=:subitems';
		Yii::app()->db->createCommand($sql)->execute(array(':subitems' => $this->id));
		
		return parent::beforeDelete();
	}

	public function beforeSave(){
		$this->page_body = str_replace('<br>', '<br/>', $this->page_body);
		if($this->isNewRecord){
			$this->active = 1;

			$maxSorter = Yii::app()->db->createCommand()
				->select('MAX(sorter) as maxSorter')
				->from('{{menu}}')
				->queryRow();
			$this->sorter = $maxSorter['maxSorter']+1;
		}
		Yii::app()->cache->delete('menu');

		if($this->special){
			if($this->subitems){
				$this->type = self::LINK_DROPDOWN_MANUAL;
			} else {
				$this->type = self::LINK_NEW_MANUAL;
			}
		}
	
		return parent::beforeSave();
	}

	public function getUrl(){
		return Yii::app()->createUrl('/menumanager/main/view', array(
			'id' => $this->id,
			'title' => $this->title,
		));
	}
	
	public static function getMenuItems(){
		$criteria = new CDbCriteria;
		$criteria->select = 'id, type, menu_title, page_title, widget, subitems, href, special';
		$criteria->condition = 'active = 1';
		$criteria->order = 'sorter';
		
		//$menu = Yii::app()->cache->get('menu');
		
		$ativeModule = $activeMenuPage = $isLinkNewAuto = $menu = false;
		
		if (Yii::app()->getController()->getModule() && Yii::app()->getController()->getModule()->getId())
			$ativeModule = Yii::app()->getController()->getModule()->getId();
		
		if ($ativeModule == 'menumanager' && Yii::app()->getController()->getActionParams()) {
			$activeMenuPage = Yii::app()->getController()->getActionParams();
			
			if ($activeMenuPage && isset($activeMenuPage['id']) && $activeMenuPage['id']) {
				$pageType = Yii::app()->db->createCommand()    
					->select('type')   
					->from(self::tableName()) 
					->where('id = '.(int) $activeMenuPage['id'])
					->queryScalar();
				
				if ($pageType == self::LINK_NEW_AUTO)
					$isLinkNewAuto = true;
			}
		}
	
		if($menu === false){
			$menuItems = self::model()->findAll($criteria);

			foreach($menuItems as $item){	
				if($item['menu_title']) {
					if($item['special']){
						if($item['id'] == 1){ // Главная страница
							$menu[] = array(
								'label' => $item['menu_title'], 'url' => array('/site/index'),
							);
						}
						else {
							$menu[] = array(
								'label' => $item['menu_title'], 
								'url' => ($item->href) ? Yii::app()->controller->createUrl($item->href) : $item->getUrl(),
								'active'=>stristr($item->href, $ativeModule),
							);
						}
					}
					else {
						if($item['type'] == self::LINK_NEW_MANUAL){
							$menu[] = array(
								'label' => $item['menu_title'],
								'url' => $item['href'],
							);
						}
						if($item['type'] == self::LINK_NEW_AUTO){
							$menu[] = array(
								'label' => $item['menu_title'],
								'url' => $item->getUrl(),
								'active' => ($ativeModule == 'menumanager' && $isLinkNewAuto) ? true : false
							);
						}

						if($item['type'] == self::LINK_DROPDOWN){
							$subitems = array();
							foreach($menuItems as $tmpItem){
								if($tmpItem['subitems'] == $item['id']){									
									if($tmpItem['type'] == self::LINK_DROPDOWN_MANUAL){
										if($tmpItem['special']){
											$tmpItem['href'] = array($tmpItem['href']);
										}
										$subitems[] = array(
											'label' => $tmpItem['menu_title'],
											'url' => $tmpItem['href'],
										);
									}
									if($tmpItem['type'] == self::LINK_DROPDOWN_AUTO){
										$subitems[] = array(
											'label' => $tmpItem['menu_title'],
											'url' => $tmpItem->getUrl(),
										);
									}
								}
							}							
							if($subitems){
								$menu[] = array(
									'label' => $item['menu_title'],
									/*'submenuOptions'=>array(
										'class'=>'sub_menu_dropdown'
									),*/
									'url'=>Yii::app()->baseUrl,
									'items' => $subitems,
									'active'=> ($ativeModule == 'menumanager' && !$isLinkNewAuto) ? true : false,
								);
							}
						}
					}
				}
			}
			Yii::app()->cache->set('menu', $menu);
		}
		if(!$menu){
			return array();
		}
		return $menu;
	}
	
	public function searchSubitems(){
		$criteria = new CDbCriteria;
		$criteria->compare('subitems', $this->id);
		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
			'pagination'=>array(
				'pageSize'=>param('adminPaginationPageSize', 20),
			),
			'sort'=>array(
				'defaultOrder'=>'sorter',
			)
		));
	}
	
	public function getTypes(){
		return array(
			self::LINK_NEW_MANUAL => 'Простая ссылка (задается вручную)',
			self::LINK_NEW_AUTO => 'Страница с текстом',
			self::LINK_DROPDOWN => 'Выпадающий список',
			self::LINK_DROPDOWN_MANUAL => 'Ссылка в выпадающем списке (задается вручную)',
			self::LINK_DROPDOWN_AUTO => 'Страница с текстом в выпадающем списке ',
		);
	}
	
	public function getForSubitems(){
		$sql = 'SELECT id, menu_title FROM {{menu}} WHERE type="'.self::LINK_DROPDOWN.'" AND active = 1';		
		$return = CHtml::listData(Yii::app()->db->createCommand($sql)->queryAll(), 'id', 'menu_title');
		
		if($this->special){
			return CMap::mergeArray(array('0' => 'Не выбрано'), $return);
		} else {
			return $return;
		}
	}
	
	public function getTitle(){
		$return = CHtml::encode($this->menu_title);
		if(!Yii::app()->user->isGuest){
			$href = array();
			switch ($this->id) {
				case 2:
					$href = array('/news/backend/main/admin');
					break;
				case 3:
					$href = array('/articles/backend/main/admin');
					break;
				case 4:
					$href = array('/gallery/backend/main/admin');
					break;
				case 5:
					$href = array('/catalog/backend/main/admin');
					break;
				case 6:
					$href = array('/price/backend/main/admin');
					break;
			}
			if($href){
				$return .= ' ['.CHtml::link('Управление разделом', $href).']';
			}
		}

		if($this->type == self::LINK_DROPDOWN){
			if(!self::model()->countByAttributes(array('subitems' => $this->id))){
				$return .= '<br/> - в данном пункте нет вложенных элементов. Пункт меню не будет отображаться.';
			}
		}

		return $return;
	}
	
	public static function getWidgetOptions(){
		$arrWidgets =  array(
			'' => 'Нет',
			'news' => 'Новости',
			'articles' => 'Статьи',
			'gallery' => 'Галерея',
			'catalog' => 'Каталог товаров',
			'price' => 'Прайс-лист',
			'contactform' => 'Форма "Свяжитесь с нами"',
		);

        if (issetModule('metrosearch')) {
            $arrWidgets['metrosearch'] = Yii::t('common', 'Search on map by metro station');
        }
        return $arrWidgets;
	}
}